function WT = WaveTransform2D(u,h,g,scale)
% returns wavelet transform of u using filter structure f
% at scale "scale"
% WT is structured
% WT.Details is a cell indexed by the cales, but WT.Details{i}
% is not a 1D  signal anymore; instead it is a structure with three fields
% the r field is the details obtained by apprying the pass band g
% on rows; it detects vertical edges
% the c field correspong to using g on columns and detects horizontal edges
% the d field applys the g filter twice and detect edges along the diagonal
%
% this implementation is not optimized in the sense that we use 8 filters
% instead of 6; on the other hand it makes straight use of conv2
WT.LoRes = u;
WT.Details = cell(scale,1); % details are stored columnwise
for i = 1:scale
    % row details
    det = myconv2D(h,g,WT.LoRes);
    WT.Details{i}.r = subsample2D(det);
    % column details
    det = myconv2D(g,h,WT.LoRes);
    WT.Details{i}.c = subsample2D(det);
    % diagonal details
    det = myconv2D(g,g,WT.LoRes);
    WT.Details{i}.d = subsample2D(det);
    % lores
    lo = myconv2D(h,h,WT.LoRes);
    WT.LoRes = subsample2D(lo);
end